package nl.kallestruik.gitlabrepostats

import io.ktor.client.*
import io.ktor.client.engine.cio.*
import io.ktor.client.plugins.contentnegotiation.*
import io.ktor.serialization.kotlinx.json.*
import kotlinx.cli.ArgParser
import kotlinx.cli.ArgType
import kotlinx.cli.default
import kotlinx.cli.required
import kotlinx.serialization.json.Json
import kotlin.time.Duration

suspend fun main(args: Array<String>) {
    val parser = ArgParser("gitlab-repo-stats")
    val gitlabUrl by parser.option(ArgType.String, fullName = "url", shortName = "u", description = "The base URL of the gitlab instance.").required()
    val accessToken by parser.option(ArgType.String, fullName = "token", shortName = "t", description = "Token used to authenticate to the gitlab api").required()
    val projectId by parser.option(ArgType.String, fullName = "project-id", shortName = "p", description = "The project ID of the repo").required()
    val pipelineStats by parser.option(ArgType.Boolean, fullName = "pipeline-stats", description = "Gather stats about the pipeline").default(false)
    val mergeStats by parser.option(ArgType.Boolean, fullName = "merge-stats", description = "Gather stats about merge requests").default(false)
    val issueStats by parser.option(ArgType.Boolean, fullName = "issue-stats", description = "Gather stats about issues").default(false)

    parser.parse(args)

    val httpClient = HttpClient(CIO) {
        install(ContentNegotiation) {
            json(Json {
                ignoreUnknownKeys = true
            })
        }
    }

    if (pipelineStats)
        PipelineStats.printStats(httpClient, gitlabUrl, accessToken, projectId)

    if (mergeStats)
        MergeStats.printStats(httpClient, gitlabUrl, accessToken, projectId)

    if (issueStats)
        IssueStats.printStats(httpClient, gitlabUrl, accessToken, projectId)
}


fun Duration.formatTime() = this.toComponents { hours, minutes, seconds, _ ->
    val hoursFormatted = hours.toString().padStart(2, '0')
    val minutesFormatted = minutes.toString().padStart(2, '0')
    val secondsFormatted = seconds.toString().padStart(2, '0')
    "$hoursFormatted:$minutesFormatted:$secondsFormatted"
}